@@ -8,12 +8,26 @@ from pydantic import BaseModel, Field, ValidationError
 
 
 class UserRole(str, Enum):
+    """Enumeration of user roles with different access levels.
+
+    Description:
+        Defines the hierarchy of user roles in the system, from limited manager
+        access to full super administrator privileges.
+    """
+
     manager = "manager"
     administrator = "administrator"
     super_administrator = "super_administrator"
 
 
 class PermissionType(str, Enum):
+    """Enumeration of permissions that can be granted to users.
+
+    Description:
+        Defines granular permissions that can be assigned to users on a per-dealer
+        basis to control access to specific features and data.
+    """
+
     dealer = "dealer"
     bulk_rule = "bulk_rule"
     inventory = "inventory"
@@ -22,7 +36,13 @@ class PermissionType(str, Enum):
 
 
 class UserData(BaseModel):
-    """Represents the authenticated user's identity, role, and access permissions."""
+    """Represents the authenticated user's identity, role, and access
+    permissions.
+
+    Description:
+        Container for user authentication data extracted from request headers,
+        including user ID, role, and dealer-specific permissions mapping.
+    """
 
     id: UUID | None = Field(default=None, description="Unique identifier of the user.")
     role: UserRole | None = Field(
@@ -35,11 +55,14 @@ class UserData(BaseModel):
     )
 
     def check_accessibility(self, dealer_id: UUID | str) -> bool:
-        """
-        Check whether the user has access rights for the given dealer.
+        """Check whether the user has access rights for the given dealer.
+
+        Description:
+            Verifies if the user can access data for a specific dealer based on
+            their role and permission mapping. Administrators have universal access.
 
         Args:
-            dealer_id (UUID | str): Dealer identifier (UUID or string).
+            dealer_id: Dealer identifier as UUID or string.
 
         Returns:
             bool: True if the user has access, False otherwise.
@@ -83,6 +106,24 @@ def get_user_data(
         ),
     ] = None,
 ) -> UserData:
+    """Extract and validate user authentication data from request headers.
+
+    Description:
+        FastAPI dependency that parses and validates user authentication headers,
+        including user ID, role, and JSON-encoded permissions mapping.
+
+    Args:
+        user_id: UUID extracted from the user-id header.
+        user_role: User role extracted from the user-role header.
+        user_permissions: JSON string of dealer permissions from user-permissions header.
+
+    Returns:
+        UserData: Validated user authentication data.
+
+    Raises:
+        HTTPException: 401 if user-id header is missing.
+        HTTPException: 400 if user-permissions JSON is malformed.
+    """
     if user_id is None:
         raise HTTPException(
             status_code=status.HTTP_401_UNAUTHORIZED, detail="Missing user-id header"
