@@ -1,6 +1,6 @@
 """Core utilities module providing foundational helpers for the framework."""
 
-from ..core.require_extra import require_extra
+from .require_extra import require_extra
 
 __all__ = (
     "async",
