@@ -1,11 +1,15 @@
+"""HTTP request utilities with error handling and logging."""
+
 from typing import Any, Dict, Optional, Union
 from urllib.parse import urljoin
 
 import requests
 
-from ..log import footprint
 from .exception import RequestException, exception_to_dict
 from .jsonable_encoder import jsonable_encoder
+from ..log import footprint
+
+__all__ = ("request",)
 
 
 def request(
@@ -23,31 +27,36 @@ def request(
     push_logs: bool = True,
     **kwargs: Any,
 ) -> Union[requests.Response, Any, str, None]:
-    controller = f"{__name__}.request"
     """Send an HTTP request with standardized headers, authentication, and
     error handling.
 
+    Description:
+        Makes HTTP requests with configurable authentication, caching headers,
+        and response handling. Supports internal service response format with
+        'success' and 'data' fields. Automatically logs errors to footprint.
+
     Args:
-        method: HTTP method (GET, POST, etc.).
+        method: HTTP method (GET, POST, PUT, DELETE, etc.).
         path: Endpoint path relative to host.
-        host: Base URL of the service.
-        auth_key: Key for authentication header or param.
+        host: Base URL of the service. Returns None if not provided.
+        auth_key: Key for authentication header or parameter.
         auth_value: Value for authentication.
-        auth_type: 'headers' or 'params'.
-        disable_caching: If True, set no-cache headers.
-        full_return: If True, return the raw Response.
-        json_return: If True, attempt to parse JSON (unless full_return).
-        internal_service: If True, expect JSON with 'success' and 'data'.
-        add_dt_user_agent: If True, include DealerTower user-agent.
-        push_logs: If True, send errors to footprint.
-        **kwargs: Passed directly to requests.request().
+        auth_type: Where to place auth - 'headers' or 'params'.
+        disable_caching: If True, sets no-cache headers.
+        full_return: If True, returns the raw Response object.
+        json_return: If True, attempts to parse JSON response.
+        internal_service: If True, expects response format with 'success' and 'data'.
+        add_dt_user_agent: If True, includes DealerTower user-agent header.
+        push_logs: If True, sends errors to footprint logging system.
+        **kwargs: Additional arguments passed directly to requests.request().
 
     Returns:
-        Depending on flags: Response, parsed JSON data, text, or None if host missing.
+        Depending on flags: Response object, parsed JSON data, text, or None.
 
     Raises:
-        RequestException: On network errors or JSON/parsing issues.
+        RequestException: On network errors, JSON parsing failures, or service errors.
     """
+    controller = f"{__name__}.request"
     if not host:
         return None
 
@@ -81,7 +90,7 @@ def request(
         headers.setdefault("User-Agent", "DealerTower-Service/1.0")
 
     # Prepare footprint context
-    error_context = {
+    error_context: Dict[str, Any] = {
         "subject": "Error sending request",
         "controller": controller,
         "payload": {
@@ -153,7 +162,7 @@ def request(
                 )
             raise RequestException(
                 status_code=status,
-                message=body.get("message"),
+                message=str(body.get("message", "")),
                 controller="dtpyfw.core.request",
             )
 
