@@ -1,12 +1,14 @@
+"""Retry utilities for functions with exponential backoff."""
+
 import asyncio
 import inspect
 import time
 from functools import wraps
-from typing import Awaitable, Callable
+from typing import Any, Awaitable, Callable, Tuple, Type, TypeVar
 
-from ..log import footprint
 from .exception import exception_to_dict
 from .jsonable_encoder import jsonable_encoder
+from ..log import footprint
 
 __all__ = (
     "retry_async",
@@ -14,18 +16,42 @@ __all__ = (
     "retry_wrapper",
 )
 
+T = TypeVar("T")
+
 
 async def retry_async(
-    func: Awaitable,
-    *args,
+    func: Callable[..., Awaitable[T]],
+    *args: Any,
     sleep_time: int | float = 2,
-    max_attempts: int | float = 3,
+    max_attempts: int = 3,
     backoff: int | float = 2,
-    exceptions=(Exception,),
+    exceptions: Tuple[Type[Exception], ...] = (Exception,),
     log_tries: bool = False,
-    **kwargs,
-):
-    """Retry an async ``func`` with exponential backoff."""
+    **kwargs: Any,
+) -> T:
+    """Retry an async function with exponential backoff.
+
+    Description:
+        Executes an async callable with automatic retry logic. On failure,
+        waits with exponentially increasing delay before retrying. Logs
+        final failures to footprint.
+
+    Args:
+        func: The async callable to execute.
+        *args: Positional arguments to pass to func.
+        sleep_time: Initial delay in seconds between retries.
+        max_attempts: Maximum number of attempts before giving up.
+        backoff: Multiplier for increasing delay after each retry.
+        exceptions: Tuple of exception types to catch and retry.
+        log_tries: If True, logs warnings for each retry attempt.
+        **kwargs: Keyword arguments to pass to func.
+
+    Returns:
+        The result from the successful function execution.
+
+    Raises:
+        Exception: The caught exception if all retry attempts fail.
+    """
     controller = f"{__name__}.retry_async"
     delay = sleep_time
     for attempt in range(1, max_attempts + 1):
@@ -54,29 +80,53 @@ async def retry_async(
                 )
             await asyncio.sleep(delay)
             delay *= backoff
+    # This should never be reached, but satisfies type checker
+    raise RuntimeError("Retry logic failed unexpectedly")
 
 
 def retry(
-    func: Callable,
-    *args,
+    func: Callable[..., T],
+    *args: Any,
     sleep_time: int | float = 2,
-    max_attempts: int | float = 3,
+    max_attempts: int = 3,
     backoff: int | float = 2,
-    exceptions=(Exception,),
+    exceptions: Tuple[Type[Exception], ...] = (Exception,),
     log_tries: bool = False,
-    **kwargs,
-):
-    """Retry a synchronous callable using the same semantics as
-    ``retry_async``."""
+    **kwargs: Any,
+) -> T:
+    """Retry a synchronous callable with exponential backoff.
+
+    Description:
+        Executes a callable with automatic retry logic. Delegates to
+        retry_async if the function is a coroutine. On failure, waits
+        with exponentially increasing delay before retrying.
+
+    Args:
+        func: The callable to execute.
+        *args: Positional arguments to pass to func.
+        sleep_time: Initial delay in seconds between retries.
+        max_attempts: Maximum number of attempts before giving up.
+        backoff: Multiplier for increasing delay after each retry.
+        exceptions: Tuple of exception types to catch and retry.
+        log_tries: If True, logs warnings for each retry attempt.
+        **kwargs: Keyword arguments to pass to func.
+
+    Returns:
+        The result from the successful function execution.
+
+    Raises:
+        Exception: The caught exception if all retry attempts fail.
+    """
     controller = f"{__name__}.retry"
     if inspect.iscoroutinefunction(func):
-        return retry_async(
+        return retry_async(  # type: ignore
             func,
             *args,
             sleep_time=sleep_time,
             max_attempts=max_attempts,
             backoff=backoff,
             exceptions=exceptions,
+            log_tries=log_tries,
             **kwargs,
         )
 
@@ -107,22 +157,40 @@ def retry(
                 )
             time.sleep(delay)
             delay *= backoff
+    # This should never be reached, but satisfies type checker
+    raise RuntimeError("Retry logic failed unexpectedly")
 
 
 def retry_wrapper(
-    max_attempts: int | float = 3,
+    max_attempts: int = 3,
     sleep_time: int | float = 2,
     backoff: int | float = 2,
-    exceptions=(Exception,),
+    exceptions: Tuple[Type[Exception], ...] = (Exception,),
     log_tries: bool = False,
-):
-    """Decorator form of :func:`retry` supporting async and sync callables."""
+) -> Callable[[Callable[..., T]], Callable[..., T]]:
+    """Decorator for retrying functions with exponential backoff.
 
-    def decorator(func: Callable):
+    Description:
+        Returns a decorator that wraps sync or async functions with retry
+        logic. Automatically detects if the decorated function is async
+        and applies the appropriate retry mechanism.
+
+    Args:
+        max_attempts: Maximum number of attempts before giving up.
+        sleep_time: Initial delay in seconds between retries.
+        backoff: Multiplier for increasing delay after each retry.
+        exceptions: Tuple of exception types to catch and retry.
+        log_tries: If True, logs warnings for each retry attempt.
+
+    Returns:
+        A decorator function that wraps callables with retry logic.
+    """
+
+    def decorator(func: Callable[..., T]) -> Callable[..., T]:
         @wraps(func)
-        async def async_wrapper(*args, **kwargs):
+        async def async_wrapper(*args: Any, **kwargs: Any) -> Any:
             return await retry_async(
-                func,
+                func,  # type: ignore
                 *args,
                 max_attempts=max_attempts,
                 sleep_time=sleep_time,
@@ -133,7 +201,7 @@ def retry_wrapper(
             )
 
         @wraps(func)
-        def sync_wrapper(*args, **kwargs):
+        def sync_wrapper(*args: Any, **kwargs: Any) -> T:
             return retry(
                 func,
                 *args,
@@ -146,7 +214,7 @@ def retry_wrapper(
             )
 
         if asyncio.iscoroutinefunction(func):
-            return async_wrapper
+            return async_wrapper  # type: ignore
         else:
             return sync_wrapper
 
