@@ -7,7 +7,18 @@ __all__ = ("create_slug",)
 
 
 def create_slug(name: str) -> str | None:
-    """Return a URL-friendly slug derived from ``name``."""
+    """Generate a URL-friendly slug from a string.
+
+    This function converts a string into a slug by lowercasing it, replacing
+    spaces with hyphens, removing special characters, and normalizing
+    diacritics.
+
+    Args:
+        name: The string to convert into a slug.
+
+    Returns:
+        The generated slug as a string, or None if the input is empty.
+    """
 
     if not name:
         return None
