@@ -10,6 +10,12 @@ from typing import Any
 from sqlalchemy import func, union
 from sqlalchemy.orm import Session
 
+from ..schema import (
+    AvailableFilterValue, AvailableFilterSelectItem, AvailableFilterNumberItem,
+    AvailableFilterDateItem, AvailableFilterSelect, AvailableFilterNumber,
+    AvailableFilterDate, AvailableFilters, AvailableFilterType,
+)
+
 __all__ = ("get_filters_value",)
 
 
@@ -19,7 +25,7 @@ def filters_mapper(
     joins: list[dict[str, Any]],
     filter_item: dict[str, Any],
     conditions: list[Any],
-) -> list[dict[str, Any]] | dict[str, Any] | None:
+) -> AvailableFilterValue:
     """Extract unique values from database columns for filter options.
 
     Queries the database to find all unique values or ranges for a given filter,
@@ -73,14 +79,14 @@ def filters_mapper(
         enum = filter_item.get("enum")
         labels = filter_item.get("labels", {})
         items = [
-            {
-                "label": labels.get(enum(value) if enum else value, value),
-                "value": enum(value).name if enum else value,
-            }
+            AvailableFilterSelectItem(
+                label=labels.get(enum(value) if enum else value, value),
+                value=enum(value).name if enum else value,
+            )
             for value in values
             if value is not None
         ]
-        items = sorted(items, key=lambda x: x.get("label") or "")
+        items = sorted(items, key=lambda x: x.label or "")
     elif columns_type in ["number", "date"]:
         if not columns:
             return None
@@ -93,11 +99,21 @@ def filters_mapper(
 
         result = main_query.filter(*pre_conditions, *conditions).first()
         if result is None:
-            range_items = {"min": None, "max": None}
+            min_value = None
+            max_value = None
         else:
             min_value, max_value = result
-            range_items = {"min": min_value, "max": max_value}
-        return range_items
+
+        if columns_type == "number":
+            return AvailableFilterNumberItem(
+                min=min_value,
+                max=max_value,
+            )
+        else:
+            return AvailableFilterDateItem(
+                min=min_value,
+                max=max_value,
+            )
     else:
         return None
 
@@ -110,7 +126,7 @@ def get_filters_value(
     joins: list[dict[str, Any]],
     filters: list[dict[str, Any]],
     names_conditions: dict[str, list[Any]],
-) -> list[dict[str, Any]]:
+) -> AvailableFilters:
     """Build a list of available filter options from database data.
 
     Queries the database for each defined filter to generate the available
@@ -125,7 +141,7 @@ def get_filters_value(
         names_conditions: Dictionary mapping filter names to their conditions.
 
     Returns:
-        List of available filter dictionaries, each containing:
+        List of available filter, each containing:
             - label: Display label for the filter
             - name: Internal filter name
             - type: Filter type (select, number, date, etc.)
@@ -136,7 +152,7 @@ def get_filters_value(
         filter_name_conditions = names_conditions.get(filter_item["name"])
         if filter_name_conditions is None:
             continue
-        filter_mapping_result = filters_mapper(
+        filter_mapping_result: AvailableFilterValue = filters_mapper(
             db=db,
             pre_conditions=pre_conditions,
             joins=joins,
@@ -149,29 +165,30 @@ def get_filters_value(
         if filter_mapping_result is not None:
             if filter_type in {"select", "select_array"}:
                 available_filters.append(
-                    {
-                        "label": filter_label,
-                        "name": filter_name,
-                        "type": filter_type,
-                        "value": filter_mapping_result,
-                    }
+                    AvailableFilterSelect(
+                        label=filter_label,
+                        name=filter_name,
+                        type=getattr(AvailableFilterType, filter_type),
+                        value=filter_mapping_result,
+                    )
                 )
-            elif filter_type in ["number", "date"]:
+            elif filter_type == "number":
                 available_filters.append(
-                    {
-                        "label": filter_label,
-                        "name": filter_name,
-                        "type": filter_type,
-                        "value": filter_mapping_result,
-                    }
+                    AvailableFilterNumber(
+                        label=filter_label,
+                        name=filter_name,
+                        type=AvailableFilterType.number,
+                        value=filter_mapping_result,
+                    )
                 )
-            else:
+            elif filter_type == "date":
                 available_filters.append(
-                    {
-                        "label": filter_label,
-                        "name": filter_name,
-                        "type": filter_type,
-                    }
+                    AvailableFilterDate(
+                        label=filter_label,
+                        name=filter_name,
+                        type=AvailableFilterType.date,
+                        value=filter_mapping_result,
+                    )
                 )
 
     return available_filters
